<?php
/**
 * PHP Mailer
 *
 * @package SMF Zhen Mailer v1.3
 * @PHP Mailer version 6.03
 * @license https://www.gnu.org/licenses/lgpl.html
 */

if (!defined('SMF'))
	die('Hacking attempt...');

function ZhenMailer_admin_areas(&$admin_areas)
{
	global $txt;

	$admin_areas['maintenance']['areas']['mailqueue']['subsections'] += array(
		'editsig' => array($txt['ZhenMailer_sig'], 'admin_forum'),
		'zhenmailset' => array($txt['ZhenMail_settings'], 'admin_forum'),
		'phpconfig' => array($txt['ZhenMail_file_config'], 'admin_forum'),
	);
}

function ZhenMailer_actions(&$actionArray)
{
	global $modSettings;

	if (!empty($modSettings['ZhenMailer_cron_job_ajax']))
		$actionArray['zhenmailerschedule'] = array('ZhenMailerCron.php', 'ZhenMailerCronJobInit');

}

function ZhenMailer_init()
{
	// load necessary language files
	global $txt, $boarddir, $sourcedir, $context;
	loadLanguage('ZhenMailer');
	require_once($sourcedir . '/ZhenMailerCron.php');
	require_once($boarddir . '/ZhenMailer/ZhenMailerFixUTF8.php');
	ZhenMailerCronJob();
}

// Send an email via SMTP.
function ZhenMailer_smtp_mail($mail_to_array, $subject, $message, $headers)
{
	global $modSettings, $webmaster_email, $txt;

	$modSettings['smtp_host'] = trim($modSettings['smtp_host']);

	// Try to connect to the SMTP server... if it doesn't exist, only wait three seconds.
	if (!$socket = fsockopen($modSettings['smtp_host'], empty($modSettings['smtp_port']) ? 25 : $modSettings['smtp_port'], $errno, $errstr, 3))
	{
		// Maybe we can still save this?  The port might be wrong.
		if (substr($modSettings['smtp_host'], 0, 4) == 'ssl:' && (empty($modSettings['smtp_port']) || $modSettings['smtp_port'] == 25))
		{
			if ($socket = fsockopen($modSettings['smtp_host'], 587, $errno, $errstr, 3))
				log_error($txt['smtp_port_ssl']);
		}

		// Unable to connect!  Don't show any error message, but just log one and try to continue anyway.
		if (!$socket)
		{
			log_error($txt['smtp_no_connect'] . ': ' . $errno . ' : ' . $errstr);
			return false;
		}
	}

	// Wait for a response of 220, without "-" continuer.
	if (!server_parse(null, $socket, '220'))
		return false;

	if ($modSettings['mail_type'] == 2 && $modSettings['smtp_username'] != '' && $modSettings['smtp_password'] != '')
	{
		// !!! These should send the CURRENT server's name, not the mail server's!
		// EHLO could be understood to mean encrypted hello...
		if (server_parse('EHLO ' . $modSettings['smtp_host'], $socket, null) == '250')
		{
			//STARTTLS
			server_parse('STARTTLS', $socket, null);
			stream_socket_enable_crypto($socket, true, STREAM_CRYPTO_METHOD_TLS_CLIENT);
			server_parse('EHLO ' . $modSettings['smtp_host'], $socket, null);

			if (!server_parse('AUTH LOGIN', $socket, '334'))
				return false;
			// Send the username and password, encoded.
			if (!server_parse(base64_encode($modSettings['smtp_username']), $socket, '334'))
				return false;
			// The password is already encoded ;)
			if (!server_parse($modSettings['smtp_password'], $socket, '235'))
				return false;
		}
		elseif (!server_parse('HELO ' . $modSettings['smtp_host'], $socket, '250'))
			return false;
	}
	else
	{
		// Just say "helo".
		if (!server_parse('HELO ' . $modSettings['smtp_host'], $socket, '250'))
			return false;
	}

	// Fix the message for any lines beginning with a period! (the first is ignored, you see.)
	$message = strtr($message, array("\r\n" . '.' => "\r\n" . '..'));

	// !! Theoretically, we should be able to just loop the RCPT TO.
	$mail_to_array = array_values($mail_to_array);
	foreach ($mail_to_array as $i => $mail_to)
	{
		// Reset the connection to send another email.
		if ($i != 0)
		{
			if (!server_parse('RSET', $socket, '250'))
				return false;
		}

		// From, to, and then start the data...
		if (!server_parse('MAIL FROM: <' . (empty($modSettings['mail_from']) ? $webmaster_email : $modSettings['mail_from']) . '>', $socket, '250'))
			return false;
		if (!server_parse('RCPT TO: <' . $mail_to . '>', $socket, '250'))
			return false;
		if (!server_parse('DATA', $socket, '354'))
			return false;
		fputs($socket, 'Subject: ' . $subject . "\r\n");
		if (strlen($mail_to) > 0)
			fputs($socket, 'To: <' . $mail_to . '>' . "\r\n");
		fputs($socket, $headers . "\r\n\r\n");
		fputs($socket, $message . "\r\n");

		// Send a ., or in other words "end of data".
		if (!server_parse('.', $socket, '250'))
			return false;

		// Almost done, almost done... don't stop me just yet!
		@set_time_limit(300);
		if (function_exists('apache_reset_timeout'))
			@apache_reset_timeout();
	}
	fputs($socket, 'QUIT' . "\r\n");
	fclose($socket);

	return true;
}

function ZhenMailer_attachments(&$attachIDs, &$msgID)
{
	global $txt, $boarddir, $smcFunc, $context, $modSettings, $sourcedir;

	if (!empty($modSettings['ZhenMailer_attachments']))
	{
		// email a copy of the attachment
		require_once($sourcedir . '/ZhenMailerCron.php');
		list($attData, $filepaths, $topicId) = array(array(), array(), 0);
		$ext = version_compare((!empty($modSettings['smfVersion']) ? substr($modSettings['smfVersion'], 0, 3) : '2.0'), '2.1', '<') ? '' : '.dat';

		foreach ($attachIDs as $attachID)
		{
			$row = array();
			$result =  $smcFunc['db_query']('', '
				SELECT att.id_attach, att.filename, att.fileext, att.id_folder, att.file_hash, att.id_msg, msg.id_topic
				FROM {db_prefix}attachments AS att
				LEFT JOIN {db_prefix}messages AS msg ON (msg.id_msg = {int:msgid})
				WHERE id_attach = {int:idatt}
				LIMIT 1',
				array(
					'idatt' => $attachID,
					'msgid' => $msgID,
				)
			);
			while ($row = $smcFunc['db_fetch_assoc']($result))
			{
				if (strlen($row['filename']) > 5 && substr($row['filename'], -6) == '_thumb')
					continue;

				$attData[] = array(
					'id' => $row['id_attach'],
					'topic' => $row['id_topic'],
					'filename' => $row['filename'],
					'fileext' => $row['fileext'],
					'file_hash' => $row['file_hash'],
					'path' => $modSettings['attachmentUploadDir'][$row['id_folder']]
				);

				$topicId = empty($topicId) ? $row['id_topic'] : $topicId;
			}

			$smcFunc['db_free_result']($result);
		}

		if (!empty($attData))
		{
			$backupDir = $boarddir . '/ZhenMailer/' . uniqidNameZhenMailer();
			@mkdir($backupDir, 0775);
			@copy($boarddir . '/ZhenMailer/index.php', $backupDir . '/index.php');
			foreach ($attData as $data)
			{
				if (file_exists($data['path'] . '/' . strval($data['id']) . '_' . $data['file_hash'] . $ext))
				{
					$filename = preg_replace('/[^a-zA-Z0-9\-\._]/', '' , $data['filename']);
					@copy($data['path'] . '/' . strval($data['id']) . '_' . $data['file_hash'] . $ext, $backupDir . '/' . $filename);
				}

				if (!file_exists($backupDir . '/' . $filename) && !empty($modSettings['ZhenMailer_log_error']))
					log_error('[3]' . $txt['ZhenMailer_move_error'] . ' ~ ' . $filename);
				else
					$filepaths[] = array(
						'filename' => $backupDir . '/' . $filename,
						'id' => $data['id']
					);
			}

			if (!empty($filepaths))
				ZhenMailerEmailAttachment($backupDir, $filepaths, $msgID, $topicId);
		}
	}
}

function ZhenMailer_manage_mail(&$subActions)
{
	global $sourcedir, $context;

	require_once($sourcedir . '/ManageZhenMailer.php');
	$subActions += array(
		'editsig' => 'ZhenMailerSig',
		'zhenmailset' => 'ZhenMailerSettings',
		'phpconfig' => 'ZhenMailerFileCreation',
	);
}

function ZhenMailerUserInfo($id, $attribute)
{
	global $smcFunc, $context, $txt, $max_views, $modSettings, $scripturl;

	list($id, $email) = array(abs($id), '');
	$att[$id] = array(
		'id_member' => 0,
		'real_name' => '',
		$attribute => ''
	);

	$result =  $smcFunc['db_query']('', '
		SELECT {raw:attr}, real_name, id_member
		FROM {db_prefix}members
		WHERE id_member = {int:memid}
		LIMIT 1',
		array(
			'memid' => $id,
			'attr' => $attribute
		)
	);

	//$att[$id] = array();

	while ($row = $smcFunc['db_fetch_assoc']($result))
	{
		$email = $row[$attribute];
		$att[$id] = array(
			'id_member' => $row['id_member'],
			'real_name' => $row['real_name'],
			$attribute => $row[$attribute]
		);
	}

	$smcFunc['db_free_result']($result);

	return $att;
}

function ZhenMailerAttachmentId($ZhenFileHash)
{
	global $smcFunc;
	list($attId, $msgId) = array(0, 0);

	// first check if the attachment was already created
	if (!empty($ZhenFileHash))
	{
		$result =  $smcFunc['db_query']('', '
			SELECT id_attach, id_msg
			FROM {db_prefix}attachments
			WHERE file_hash = {string:fh}
			LIMIT 1',
			array(
				'fh' => $ZhenFileHash
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($result))
		{
			$attId = !empty($row['id_attach']) ? $row['id_attach'] : 0;
			$msgId = !empty($row['id_msg']) ? $row['id_msg'] : 0;
		}

		$smcFunc['db_free_result']($result);
	}

	// hash has not been created yet therefore figure out the last increment of attachment id
	if (empty($attId))
	{
		$result =  $smcFunc['db_query']('', '
			SELECT id_attach, id_msg
			FROM {db_prefix}attachments
			ORDER BY id_attach DESC
			LIMIT 1',
			array(
			)
		);
		while ($row = $smcFunc['db_fetch_assoc']($result))
		{
			$attId = !empty($row['id_attach']) ? $row['id_attach'] : 0;
			$msgId = !empty($row['id_msg']) ? $row['id_msg'] : 0;
		}

		$smcFunc['db_free_result']($result);
	}

	// message id has not been created yet therefore figure out the next increment
	if (empty($msgId))
	{
		$result =  $smcFunc['db_query']('', '
			SHOW TABLE STATUS LIKE "{db_prefix}messages"',
			array()
		);
		while ($row = $smcFunc['db_fetch_assoc']($result))
			$msgId = !empty($row['Auto_increment']) ? $row['Auto_increment'] : '';

		$smcFunc['db_free_result']($result);

		$msgId = (int)$msgId;
	}

	return array($attId, $msgId);
}

?>