<?php
if (!defined('SMF'))
	die('Hacking attempt...');

/**
 * Zhen Mailer
 *
 * @package SMF Zhen Mailer v1.3
 * @PHP Mailer version 6.03
 * @license https://www.gnu.org/licenses/lgpl.html
 */

use \ForceUTF8\Encoding;

function ZhenMailerSMF($toArray, $subject, $encoded_message, $html_message, $sendhtml, $char_set, $is_private, $filesArray)
{
	global $modSettings, $txt, $mbname, $webmaster_email, $context;
	$toArray = !is_array($toArray) ? array($toArray) : $toArray;
	$userDefault = !empty($modSettings['ZhenMail_user_default']) ? $modSettings['ZhenMail_user_default'] : 0;
	list(
		$mail_result,
		$send_html,
		$modSettings['smtp_host'],
		$subject,
		$from,
		$to_array,
		$email_domain_name,
		$line_break,
		$charset,
		$signature,
		$files,
		$port
	) = array (
		false,
		!empty($sendhtml) ? true : false,
		trim($modSettings['smtp_host']),
		strtr($subject, array("\r" => '', "\n" => '')),
		empty($modSettings['mail_from']) ? $webmaster_email : $modSettings['mail_from'],
		array_values(array_unique($toArray)),
		strpos($webmaster_email, '@') !== false ? 'members@' . substr(strrchr($webmaster_email, "@"), 1) : 'members@example.com',
		($context['server']['is_windows'] || version_compare(PHP_VERSION, '8.0.0', '>=')) ? "\r\n" : "\n",
		!empty($char_set) ? $char_set : 'UTF-8',
		!empty($modSettings['smtp_signature']) ? $modSettings['smtp_signature'] : '',
		!empty($filesArray) ? $filesArray : array(),
		!empty($modSettings['smtp_port']) ? (int)$modSettings['smtp_port'] : 587
	);

	$recipients = count($to_array);
	$files = !is_array($files) ? array($files) : $files;
	$sig = preg_replace("~([^\.]{4,}?)(\r\n|[\r\n]|$)~", "$1.\r\n", $signature);
	$sig = preg_replace('/<br(\s+)?\/?>/i', "\r\n", $sig);

	if (!empty($userDefault) && count($to_array) == 1 && $to_array[0] == $webmaster_email)
	{
		$data = ZhenMailerUserInfo($userDefault, 'email_address');
		$to_array[] = $data[$userDefault]['email_address'];
	}


	//Create a new PHPMailer instance
	$mail = new PHPMailer(true);

	try
	{
		if (!empty($modSettings['mail_type']))
		{
			$mail->SMTPDebug = 0;
			$mail->isSMTP();
			$mail->Host = $modSettings['smtp_host'];
			$mail->SMTPAuth = true;
			$mail->Username = $modSettings['smtp_username'];
			$mail->Password = base64_decode($modSettings['smtp_password']);
			/*
			if ($modSettings['mail_type'] == 2)
				$mail->SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS;
			else
				$mail->SMTPSecure = PHPMailer::ENCRYPTION_SMTPS;
			*/
			$mail->SMTPSecure = 'tls';
			$mail->Port = intval($port);
			$mail->setFrom($modSettings['smtp_username'], $mbname);
		}
		else
		{
			//$mail->SMTPAutoTLS = true;
			$mail->setFrom($webmaster_email, $mbname);
		}

		$mail->CharSet = $charset;
		foreach ($files as $file)
		{
			if (file_exists($file))
				$mail->addAttachment($file);
		}

		foreach ($to_array as $to)
		{
			$sendto = strtr($to, array("\r" => '', "\n" => ''));

			if ($recipients == 1)
				$mail->addAddress($sendto);
			else
				$mail->AddBCC($sendto);
		}
		if ($recipients > 1)
			$mail->addAddress($email_domain_name);

		if (empty($is_private))
			$mail->addReplyTo($from, $mbname);

		//Content
		// Set email format to HTML
		if ($send_html)
		{
			$mail->isHTML(true);
			$mail->ContentType = 'text/html';
			$mail->Encoding = "8bit";
			$msg = trim(ZhenMailerEntities($html_message, false)) . "<br>" . $sig . "<br>";
			//$mail->MsgHTML($msg);
			$msg = str_replace("\r\n", "<br>", $msg);
			$mail->Body = trim($msg);
			$mail->AltBody = trim($encoded_message) . "\r\n" . trim($sig) . "\r\n";
		}
		else
		{
			$mail->isHTML(false);
			$mail->ContentType = 'text/plain';
			$mail->Encoding = "8bit";
			//$safe = filter_var($encoded_message, FILTER_SANITIZE_STRING);
			$msg = trim(ZhenMailerEntities($encoded_message, false)) . "\r\n" . $sig . "\r\n";
			$msg = str_ireplace(array("<br>", "<br />"), array("\r\n", "\r\n"), $msg);
			$mail->Body = trim($msg);
		}
		$mail->Subject = $subject;

		$mail->send();
		$mail_result = true;
	}
	catch (Exception $e)
	{
		$toList = '';
		foreach ($to_array as $to)
			$toList .= ' ' . $to . ',';
		$toList = rtrim($toList, ',');
		$toList = trim($toList);
		if (!empty($modSettings['ZhenMailer_log_error']))
			log_error('[2.4] ' . sprintf($txt['mail_send_unable'], $toList));
		$mail_result = false;
	}
	$mail->clearAllRecipients();

	return $mail_result;
}

function ZhenMailerEntities($str, $flag)
{
	//$str = base64_decode($str) == $str ? $str : base64_decode($str);
	// smf seems to encode any message with special chars as base64
	if (!$flag)
		return $str;

    list($str, $str2) = array(html_entity_decode(stripslashes($str),ENT_QUOTES,'UTF-8'), '');
    $ar = preg_split('/(?<!^)(?!$)/u', $str );
    foreach ($ar as $c)
	{
        $o = ord($c);
        if ((strlen($c) > 1) || ($o <32 || $o > 126) || ($o >33 && $o < 40) || ($o >59 && $o < 63))
			$c = mb_encode_numericentity($c,array (0x0, 0xffff, 0, 0xffff), 'UTF-8');

        $str2 .= $c;
    }
    return $str2;
}

function ZenMailerFixBrokenUTF8($stringReturn)
{
	global $modSettings;

	return !empty($modSettings['ZhenMail_fix_double']) ? Encoding::fixUTF8($stringReturn) : $stringReturn;
}

?>